def write_optuna_execution_script(path, inputs, input_branch_name):
    with open(path,'w') as f:

        print('# !/usr/bin/env python', file=f)
        print('# coding: utf-8', file=f)
        print('', file=f)

        print('def run_experiment(trial):', file=f)
        print('    from dvc_cc.hyperopt.variable import VariableCache', file=f)
        print('    import numpy as np', file=f)
        print('    import time', file=f)
        print('    vc = VariableCache()', file=f)
        print("    list_of_hyperopt_files = [f for f in os.listdir(str(Path('dvc/.hyperopt'))) if f.endswith('.hyperopt')]", file=f)
        print('    for f in list_of_hyperopt_files:', file=f)
        print("        f = str(Path('dvc/.hyperopt/' + f))", file=f)
        print('        vc.register_dvccc_file(f)', file=f)
        print('', file=f)

        print('    values = []', file=f)
        for k in inputs:
            t,v = inputs[k]
            if v.find('-') > 0:
                if t == 'float':
                    print('    values.append(trial.suggest_uniform("'+k+'", '+v[:v.find('-')]+','+v[v.find('-')+1:]+')'+')', file=f)
                else:
                    print('    values.append(trial.suggest_int("'+k+'", '+v[:v.find('-')]+','+v[v.find('-')+1:]+')'+')', file=f)
            elif v.find(',') > 0:
                print('    values.append(trial.suggest_categorical("'+k+'", ['+v+'])'
                      +')', file=f)
            else:
                print('    values.append('+v+') // ' + k, file=f)

        print("    result_branch = '"+input_branch_name+"_'+str(time.time_ns())", file=f)
        print("    dvc_save_path = 'dvc/' + result_branch", file=f)
        print('    os.mkdir(dvc_save_path)', file=f)
        print('    vc.set_values_for_hyperopt_files(values, dvc_save_path=dvc_save_path)', file=f)
        print('', file=f)
        print('    # Create RED-YMLnano', file=f)
        print("    with open('cc_execution_file_optuna.red.yml','r') as f:", file=f)
        print('        red_yml = f.read()', file=f)
        print("    red_yml = red_yml.replace('rcc_0032_MetricTest_XXXXXXXXXX', result_branch)", file=f)
        print("    with open(dvc_save_path+'/cc_execution_file.red.yml','w') as f:", file=f)
        print('        print(red_yml, file=f)', file=f)
        print('', file=f)
        print("    subprocess.call(('git add '+dvc_save_path+'/*').split(' '))", file=f)
        print('    subprocess.call("git commit -m \'SOMETHING\'".split(\' \'))', file=f)
        print('    subprocess.call("git push".split(\' \'))', file=f)
        print('', file=f)
        print('    # START RUN', file=f)
        print("    p = 'faice exec '+dvc_save_path+'/cc_execution_file.red.yml'# --disable-retry'", file=f)
        print("    message = subprocess.call(p.split(' '))", file=f)
        print('    result = None', file=f)
        print('    while result is None:', file=f)
        print('        time.sleep(np.random.randint(60))', file=f)
        print("        with open('metrics.json', 'r') as f:", file=f)
        print('            metrics = json.load(f)', file=f)
        print("        metrics = {m[:m.find('__')]:metrics[m] for m in metrics}", file=f)
        print('        if result_branch in metrics:', file=f)
        print('            result = metrics[result_branch]', file=f)
        print('', file=f)
        print('    return result', file=f)
        print('', file=f)
        print('import optuna', file=f)
        print('', file=f)
        print("study = optuna.create_study(direction='minimize',", file=f)
        print("                            study_name='study_1', ", file=f)
        print("                            storage='sqlite:///"+input_branch_name+".db?timeout=99999',", file=f)
        print('                            load_if_exists=True', file=f)
        print('                           )', file=f)
        print('', file=f)
        print('study.optimize(startbla, n_trials=300,n_jobs=150)', file=f)


# parameters
#metric_path = 'loss.metric'
#input_branch = 'cc_0032_MetricTest'
#save_summary = '../learn-subjectivity/metrics.json'
def write_metric_getter_script(path, input_branch_name, metric_path, save_summary):
    with open(path,'w') as f:
        print('# !/usr/bin/env python', file=f)
        print('# coding: utf-8', file=f)
        print('', file=f)
        print('import git\nimport json\nimport os\nimport time\n', file=f)
        print('', file=f)
        print('g = git.Git()', file=f)
        print('', file=f)
        print("metric_path = '"+metric_path+"'", file=f)
        print("input_branch = '"+input_branch_name+"'", file=f)
        print("save_summary = '" + save_summary + "'", file=f)
        print('', file=f)
        print('metrics = {}', file=f)
        print('if not os.path.isfile(save_summary):', file=f)
        print("    with open(save_summary, 'w') as f:", file=f)
        print('        json.dump(metrics, f)', file=f)
        print('else:', file=f)
        print("    with open(save_summary, 'r') as f:", file=f)
        print('        metrics = json.load(f)', file=f)
        print('', file=f)
        print('while True:', file=f)
        print('    g.pull()', file=f)
        print('    time.sleep(1)', file=f)
        print("    all_branches = [b.split('/')[-1] for b in g.branch('-a').split() if", file=f)
        print("                    b.startswith('remotes/origin/r' + input_branch)]", file=f)
        print('    new_branches = [b for b in all_branches if b not in metrics.keys()]', file=f)
        print('    for b in new_branches:', file=f)
        print('        g.checkout(b)', file=f)
        print("        with open(metric_path, 'r') as f:", file=f)
        print('            metrics[b] = float(f.read())', file=f)
        print("    with open(save_summary, 'w') as f:", file=f)
        print('        json.dump(metrics, f)', file=f)
        print('    if len(new_branches):', file=f)
        print("        print('New-Branches: ', new_branches)", file=f)
        print('', file=f)



def create_optuna_directories(input_branch_name, vc):
    # 1. copy current directory
    import os
    import shutil
    jobstarter_path = '../'+os.getcwd().split('/')[-1]+'_'+input_branch_name.split('_')[1]+'_JobStarter'
    shutil.copytree('.', jobstarter_path)

    metric_path = '../' + os.getcwd().split('/')[-1] + '_MetricGetter'
    if not os.path.exists(metric_path):
        shutil.copytree('.', jobstarter_path)

    # 2. get input values
    inputs = {}
    for v in vc.list_of_all_variables:
        inp = input(str(v.varname + ' (' + v.vartype + '): '))
        inputs[v.varname] = [v.vartype, inp]

    # 3. Create optuna script
    write_optuna_execution_script(path=jobstarter_path+'/run_optuna.py',inputs=inputs, input_branch_name=input_branch_name)

    # 4. Create metrics getter script
    # TODO: The 'loss.metric' must be a variable !
    write_metric_getter_script(metric_path + '/get_metrics.py', input_branch_name,
                               'loss.metric', jobstarter_path+'/metrics.json')


import sys
from git import Repo as GITRepo
from dvc_cc.hyperopt.variable import *
import subprocess
import uuid
import os
import numpy as np
from dvc_cc.bcolors import *
from pathlib import Path
from dvc_cc.run.jupyter_notebook_to_source import jupyter_notebook_to_source
DESCRIPTION = 'This command guesses the "dvc-cc hyperopt new" commands that might interest you. ' \
              'This is a simple implementation that you must review and correct.'

def search_for_files(command, preference = '-d'):
    command = command.split('#')[0].replace('"','\'').split('\'')

    if len(command) < 2:
        return None

    # is open function:
    if command[0].find(' open(') >= 2 or command[0].find('=open(') >= 1:
        founded_file = ''
        if len(command) >= 5:
            if len(command[3]) < 4 and (command[3].find('w') >= 0 or command[3].find('a') >= 0):
                if command[1].endswith('.yml') or command[1].endswith('.json'):
                    founded_file = '-m ' + command[1]
                else:
                    founded_file = '-o ' + command[1]
        if founded_file == '':
            return '-d ' + command[1]
        else:
            return founded_file
    elif command[0].find('ModelCheckpoint(') >= 0:
        # maybe a tf.keras.callbacks.ModelCheckpoint call?
        for i in range(1, len(command), 2):
            if command[i].endswith('.h5'):
                return '-o ' + command[i]

    elif command[0].find('TensorBoard(') >= 0:
        # maybe a tf.keras.callbacks.TensorBoard call?
        return '-o ' + command[1]

    for i in range(1,len(command),2):
        if command[i].endswith('.yml') or command[1].endswith('.json'):
            return '-m ' + command[1]

        elif command[i].endswith('.csv') or command[1].endswith('.h5') \
                or command[1].endswith('.txt') or command[1].endswith('.rtf')\
                or command[1].endswith('.bmp') or command[1].endswith('.jpg')\
                or command[1].endswith('.jpeg') or command[1].endswith('.gif')\
                or command[1].endswith('.html') or command[1].endswith('.mp3')\
                or command[1].endswith('.mpg') or command[1].endswith('.mpeg')\
                or command[1].endswith('.avi') or command[1].endswith('.wmf')\
                or command[1].endswith('.mov') or command[1].endswith('.ram')\
                or command[1].endswith('.tif') or command[1].endswith('.tiff')\
                or command[1].endswith('.dat') or command[1].endswith('.js')\
                or command[1].endswith('.mp4') or command[1].endswith('.tmp'):
            return preference + ' ' + command[1]

    return None

def search_for_argparse_parameter(command):
    if command.find('.add_argument(\'') > 0:

        params = command.split('.add_argument(')[1].split(',')

        if params[0].startswith('\'--'): # optional parameter but only with long name
            shortname = params[0].strip()[3:-1].replace('-','').replace('_','')
            longname = params[0].strip()[1:-1]
        elif params[0].startswith('\'-'): # required parameter
            shortname = params[0].strip()[2:-1]
            longname = params[1].strip()[1:-1]
        else:
            shortname = params[0].strip()[1:-1]
            longname = ''
        dtype = ''
        for p in params[1:]:
            p = p.strip()
            if p.startswith('type=') or p.startswith('type ='):
                if p[5:].startswith('int'):
                    dtype = 'int'
                elif p[5:].startswith('float'):
                    dtype = 'float'

            if p.startswith('default='):
                p = p[8:]
                if p[0] == '\'':
                    p = p[1:p[1:].find('\'')]
                if p[-1] == ')':
                    p = p[:-1]
                print('dvc-cc hyperopt var --set ' + p + ' ' + shortname)


        var = '{{' + shortname
        if dtype != '':
            var = var + ':' + dtype
        var = var + '}}'
        if longname == '':
            return var
        else:
            return longname + ' ' + var
    else:
        None

def main():
    for (dirpath, dirnames, filenames) in os.walk('.'):
        for filename in filenames:
            content = []
            path = dirpath+'/'+filename
            if filename.endswith('.py') and dirpath.find('.ipynb_checkpoints') == -1:
                with open(str(Path(path))) as f:
                    content = f.readlines()
            elif filename.endswith('.ipynb') and dirpath.find('.ipynb_checkpoints') == -1:
                path = path[:-6] + '.py'
                content = jupyter_notebook_to_source(dirpath,filename).split('\n')

            # this variable is used to save lines, if a command goes over several lines.
            saved_lines = ''

            file_contains_argparser = False
            params = []
            outputs = []

            # for each line
            for i in range(len(content)):
                line = content[i].replace('"',"'")
                splitted_line = line.split()

                if len(splitted_line) >= 2 and splitted_line[1] == 'argparse':
                    file_contains_argparser = True

                line = line.strip().split('#')[0]

                #f len(saved_lines) > 0:
                saved_lines = saved_lines + line
                line = saved_lines

                if line.count('(') == line.count(')'):
                    saved_lines = ''
                    # the command is complete
                    param = search_for_argparse_parameter(line)
                    if param is not None:
                        params.append(param)

                    if i  < 0.6 * len(content):
                        output = search_for_files(line,preference='-d')
                    else:
                        output = search_for_files(line,preference='-o')
                    if output is not None:
                        outputs.append(output)

            if file_contains_argparser:
                #params = list(np.unique(params))
                indexes = np.unique(params, return_index=True)[1]
                params = [params[index] for index in sorted(indexes)]

                #outputs = list(np.unique(outputs))
                indexes = np.unique(outputs, return_index=True)[1]
                outputs = [outputs[index] for index in sorted(indexes)]


                if filename.endswith('.py'):
                    filename = filename[:-3]+'.dvc'
                elif filename.endswith('.ipynb'):
                    filename = filename[:-6]+'.dvc'
                print('dvc-cc hyperopt new -d ' + path[2:] + ' \\')
                for output in outputs:
                    print('                    '+output+' \\')
                print('                    -f '+filename+' \\')
                print('                    \'python '+path[2:]+' '+' '.join(params)+'\'')
                print()
    print()
    print(bcolors.HEADER+'This is a simple implementation that you must review and correct.'+bcolors.ENDC)